/*
 * mpu6050.c
 *
 *  Created on: Mar 2, 2016
 *      Author: gerik
 */

#include "xparameters.h"
#include "xiic.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "mpu6050.h"
#include <math.h>

#define MPU6050_IIC_DEVICE_ID	XPAR_AXI_IIC_0_DEVICE_ID

#define MPU6050_ADDRESS	0x68

static XIic mpu6050Iic;
static XIic_Config* config;

int initMPU6050()
{
	config = XIic_LookupConfig(MPU6050_IIC_DEVICE_ID);

	XIic_CfgInitialize(&mpu6050Iic, config, config->BaseAddress);

	u8 sleepConfig[] = {107, 0x01};

	XIic_Send(config->BaseAddress, MPU6050_ADDRESS,
			  sleepConfig, 2, 0);

	u8 gyroConfig[] = {0x1B, 0x08};

	XIic_Send(config->BaseAddress, MPU6050_ADDRESS,
			  gyroConfig, 2, 0);

	return 0;
}

imuData getIMUData()
{
	u8 getAccelCmd[] = {0x3B};
	u8 getGyroCmd[] = {0x43};
	u8 recvData[6];


	imuData retData;

	XIic_Send(config->BaseAddress, MPU6050_ADDRESS,
			  getAccelCmd, 1, 0);

	XIic_Recv(config->BaseAddress, MPU6050_ADDRESS,
			  recvData, 6, 0);

	retData.accelX = recvData[0] << 8 | recvData[1];
	retData.accelY = recvData[2] << 8 | recvData[3];
	retData.accelZ = recvData[4] << 8 | recvData[5];

	XIic_Send(config->BaseAddress, MPU6050_ADDRESS,
			  getGyroCmd, 1, 0);

	XIic_Recv(config->BaseAddress, MPU6050_ADDRESS,
			  recvData, 6, 0);

	retData.gyroX = recvData[0] << 8 | recvData[1];
	retData.gyroY = recvData[2] << 8 | recvData[3];
	retData.gyroZ = recvData[4] << 8 | recvData[5];

	return retData;
}

void imuPollTask(void* dummy)
{
	imuData dataPoints[5];
	imuData offset = {444, 253, -1450, -321 / 2, 198 / 2, 15 / 2};
	imuData filteredData;
	int i;
	int32_t temp[6] = {0};
	char writeOutData[256];

//	for (i = 0; i < 5; i++)
//	{
//		vTaskDelay(configTICK_RATE_HZ/100);
//
//		offsets[i] = getIMUData();
//	}
//
//	for (i = 0; i < 5; i++)
//	{
//		temp[0] += offsets[i].accelX;
//		temp[1] += offsets[i].accelY;
//		temp[2] += offsets[i].accelZ;
//		temp[3] += offsets[i].gyroX;
//		temp[4] += offsets[i].gyroY;
//		temp[5] += offsets[i].gyroZ;
//	}
//
//	offset.accelX = temp[0] / 5;
//	offset.accelY = temp[1] / 5;
//	offset.accelZ = temp[2] / 5;
//	offset.gyroX = temp[3] / 5;
//	offset.gyroY = temp[4] / 5;
//	offset.gyroZ = temp[5] / 5;
//
//	offset.accelZ -= 16384;

	setDutyCycle(0);

	vTaskDelay(configTICK_RATE_HZ);

	while (1)
	{
		memset(temp, 0, sizeof(temp));
		for (i = 0; i < 5; i++)
		{
			vTaskDelay(configTICK_RATE_HZ / 200);

			dataPoints[i] = getIMUData();

			dataPoints[i].accelX -= offset.accelX;
			dataPoints[i].accelY -= offset.accelY;
			dataPoints[i].accelZ -= offset.accelZ;
			dataPoints[i].gyroX -= offset.gyroX;
			dataPoints[i].gyroY -= offset.gyroY;
			dataPoints[i].gyroZ -= offset.gyroZ;
		}

		for (i = 0; i < 5; i++)
		{
			temp[0] += dataPoints[i].accelX;
			temp[1] += dataPoints[i].accelY;
			temp[2] += dataPoints[i].accelZ;
			temp[3] += dataPoints[i].gyroX;
			temp[4] += dataPoints[i].gyroY;
			temp[5] += dataPoints[i].gyroZ;
		}

		filteredData.accelX = temp[0] / 5;
		filteredData.accelY = temp[1] / 5;
		filteredData.accelZ = temp[2] / 5;
		filteredData.gyroX = temp[3] / 5;
		filteredData.gyroY = temp[4] / 5;
		filteredData.gyroZ = temp[5] / 5;

		// Write out filtered Data
		float angle = asin(filteredData.accelX / 16384);
		float KVals[] = {-1000, -3.128, -1.0036};
		float bodySpeed = (filteredData.gyroZ / 65.5) * 2 * 3.14;


		float controlOut = KVals[0] * angle +
						   KVals[1] * bodySpeed +
						   KVals[2] * readSpeed();

		controlOut = (controlOut + 20) / 5;

		char outBuf[32];
		sprintf(outBuf, "%d\r\n", (int)controlOut);
		uartWrite(outBuf);

		setDutyCycle((int)controlOut | 0x80);

		//memset(writeOutData, 0, 256);
//		sprintf(writeOutData, "X: %d Y: %d Z: %d\r\ngX: %d gY: %d gZ: %d\r\n",
//				filteredData.accelX, filteredData.accelY, filteredData.accelZ,
//				filteredData.gyroX, filteredData.gyroY, filteredData.gyroZ);
//		sprintf(writeOutData, "%d\r\n", filteredData.gyroZ);
		//uartWrite(writeOutData);
	}
}

