/*
 * uartRead.c
 *
 *  Created on: Feb 28, 2016
 *      Author: gkubiak
 */

#include <string.h>

#include "xparameters.h"
#include "xuartps.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#define UART_DEVICE_ID              XPAR_PS7_UART_1_DEVICE_ID

QueueHandle_t uartReadQueue = NULL;

static XUartPs Uart_PS;

static int initUart()
{
	XUartPs_Config *Config;
	int Status;

	Config = XUartPs_LookupConfig(UART_DEVICE_ID);
	if (NULL == Config) {
		return XST_FAILURE;
	}

	Status = XUartPs_CfgInitialize(&Uart_PS, Config, Config->BaseAddress);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	XUartPs_SetBaudRate(&Uart_PS, 115200);

	return XST_SUCCESS;
}

//Reads UART input and places it in the read queue
void uartReadTask( void *pvParameters )
{
	if (initUart() == XST_FAILURE)
	{
		//Error with init
	}

	while (1)
	{
		char data;

		while (XUartPs_Recv(&Uart_PS, &data, 1))
		{
			xQueueSend( uartReadQueue, &data, 0U );
		}

		vTaskDelay(10);
	}
}

void uartWrite(char* string)
{
	XUartPs_Send(&Uart_PS, string, strlen(string));
}

void uartWriteChar(char data)
{
	XUartPs_Send(&Uart_PS, &data, 1);
}
